@@ -53,7 +53,6 @@ static opkg_option_t options[] = {
     {"lock_file", OPKG_OPT_TYPE_STRING, &_conf.lock_file},
     {"info_dir", OPKG_OPT_TYPE_STRING, &_conf.info_dir},
     {"status_file", OPKG_OPT_TYPE_STRING, &_conf.status_file},
-    {"force_defaults", OPKG_OPT_TYPE_BOOL, &_conf.force_defaults},
     {"force_maintainer", OPKG_OPT_TYPE_BOOL, &_conf.force_maintainer},
     {"ignore_maintainer", OPKG_OPT_TYPE_BOOL, &_conf.ignore_maintainer},
     {"ignore_uid", OPKG_OPT_TYPE_BOOL, &_conf.ignore_uid},
@@ -72,7 +71,6 @@ static opkg_option_t options[] = {
     {"http_proxy", OPKG_OPT_TYPE_STRING, &_conf.http_proxy},
     {"https_proxy", OPKG_OPT_TYPE_STRING, &_conf.https_proxy},
     {"no_proxy", OPKG_OPT_TYPE_STRING, &_conf.no_proxy},
-    {"test", OPKG_OPT_TYPE_BOOL, &_conf.noaction},
     {"noaction", OPKG_OPT_TYPE_BOOL, &_conf.noaction},
     {"download_only", OPKG_OPT_TYPE_BOOL, &_conf.download_only},
     {"download_first", OPKG_OPT_TYPE_BOOL, &_conf.download_first}, /* Not available on internal solver */
@@ -182,7 +180,7 @@ static void parse_pkg_src_options_str(pkg_src_options_t *src_options,
     char *token, *value, *src_option;
 
     /* default value */
-    src_options->disable_sig_check = 0;
+    src_options->signature_verified = 0;
 
     token = strtok(options_str, " ");
     while (token) {
@@ -193,9 +191,9 @@ static void parse_pkg_src_options_str(pkg_src_options_t *src_options,
             src_option = xstrndup(token, strlen(token) - (strlen(value) + 1));
             if (strcasecmp(src_option, "trusted") == 0) {
                 if (strcasecmp(value, "yes") == 0)
-                    src_options->disable_sig_check = 1;
+                    src_options->signature_verified = 1;
                 else
-                    src_options->disable_sig_check = 0;
+                    src_options->signature_verified = 0;
             }
             free(src_option);
         }
@@ -604,7 +602,7 @@ int opkg_conf_init(void)
     return 0;
 }
 
-static int opkg_lock()
+int opkg_lock()
 {
     int r;
     char *lock_dir;
@@ -643,7 +641,7 @@ static int opkg_lock()
     return 0;
 }
 
-static int opkg_unlock()
+int opkg_unlock()
 {
     int r;
     int err = 0;
@@ -668,6 +666,8 @@ static int opkg_unlock()
         if (r == -1) {
             opkg_perror(ERROR, "Couldn't unlink %s", opkg_config->lock_file);
             err = -1;
+        } else {
+            lock_fd = -1;
         }
     }
 
@@ -759,7 +759,6 @@ int opkg_conf_load(void)
         opkg_config->lock_file = tmp;
     }
 
-    r = opkg_lock();
     if (r < 0)
         goto err2;
 
@@ -870,13 +869,6 @@ int opkg_conf_load(void)
         opkg_config->cache_dir = tmp;
     }
 
-    r = file_mkdir_hier(opkg_config->cache_dir, 0755);
-    if (r != 0) {
-        opkg_perror(ERROR, "Creating cache dir %s failed",
-                    opkg_config->cache_dir);
-        goto err4;
-    }
-
     r = resolve_pkg_dest_list();
     if (r != 0)
         goto err4;
@@ -894,8 +886,6 @@ int opkg_conf_load(void)
     if (r == -1)
         opkg_perror(ERROR, "Couldn't remove dir %s", opkg_config->tmp_dir);
  err3:
-    opkg_unlock();
-
  err2:
  err1:
     pkg_src_list_deinit(&opkg_config->pkg_src_list);
@@ -951,8 +941,6 @@ void opkg_conf_deinit(void)
     hash_table_deinit(&opkg_config->file_hash);
     hash_table_deinit(&opkg_config->obs_file_hash);
 
-    opkg_unlock();
-
     for (i = 0; options[i].name; i++) {
         if (options[i].type == OPKG_OPT_TYPE_STRING) {
             tmp = (char **)options[i].value;
